/********************************************************************************
 This function finds E.R.I.C.'s foot positions (XYZ co-ordinates)
 Notebook: http://mbed.org/users/ms523/notebook/robodog-leg-forward-kinematics/
********************************************************************************/
#include "ERIC.h"

Positions FK_Engine(){
    
    // Read the ax-12+ Servos and translate into joint angles alpha, and phi for each leg
    float hind_right_alpha = 165 - Right_Hip_Lateral.GetPosition();
    float hind_right_theta = 340 - Right_Hip.GetPosition();
    float hind_right_phi = 320 - Right_Knee.GetPosition();
    float hind_left_alpha = Left_Hip_Lateral.GetPosition() - 135;
    float hind_left_theta = Left_Hip.GetPosition() + 40;
    float hind_left_phi = Left_Knee.GetPosition() + 20;    
    float temp;                             // used to store maths
    
    // Convert into radians for maths
    hind_right_alpha = hind_right_alpha * PI / 180;
    hind_right_theta = hind_right_theta  * PI / 180;
    hind_right_phi = hind_right_phi * PI / 180;
    hind_left_alpha = hind_left_alpha * PI / 180;
    hind_left_theta = hind_left_theta  * PI / 180;
    hind_left_phi = hind_left_phi * PI / 180;

    // Next calculate point 1 for both legs - start with the right leg...
    float x1_right = sin(hind_right_alpha);
    x1_right = x1_right * B;
    float y1_right = 0;
    float z1_right = cos(hind_right_alpha);
    z1_right = z1_right * B;
    // And for the left leg
    float x1_left = sin(hind_left_alpha + PI);  //Add PI to invert to get the sign correct
    x1_left = x1_left * B;            
    float y1_left = 0;
    float z1_left = cos(hind_left_alpha);
    z1_left = z1_left * B;


    // Now calculate point 2 - again start with the right leg..
    float x2_right = cos(hind_right_theta + PI);
    x2_right = x2_right * E;
    temp = cos(hind_right_alpha);
    x2_right = x2_right * temp;
    x2_right = x1_right + x2_right;
    float y2_right = sin(hind_right_theta);
    y2_right = y2_right * E;
    y2_right = y2_right + y1_right;
    float z2_right = cos(hind_right_theta);
    z2_right = z2_right * E;
    temp = sin(hind_right_alpha);
    z2_right = z2_right * temp;
    z2_right = z1_right + z2_right;
    // And now the left leg...
    float x2_left = cos(hind_left_theta + PI);  
    x2_left = x2_left * E;
    temp = cos(hind_left_alpha + PI);       //Add PI to invert to get the sign correct
    x2_left = x2_left * temp;
    x2_left = x1_left + x2_left;
    float y2_left = sin(hind_left_theta);
    y2_left = y2_left * E;
    y2_left = y2_left + y1_left;
    float z2_left = cos(hind_left_theta);
    z2_left = z2_left * E;
    temp = sin(hind_left_alpha);
    z2_left = z2_left * temp;
    z2_left = z1_left + z2_left;

    // Finally calculate point 3 - start with the right leg..
    float x3_right = cos(hind_right_theta - hind_right_phi);
    x3_right = x3_right * F;
    temp = cos(hind_right_alpha);
    x3_right = x3_right * temp;
    x3_right = x2_right + x3_right;
    float y3_right = sin(hind_right_theta + PI - hind_right_phi);
    y3_right = y3_right * F;
    y3_right = y2_right + y3_right;
    float z3_right = cos(hind_right_theta + PI - hind_right_phi);
    z3_right = z3_right * F;
    temp = sin(hind_right_alpha);
    z3_right = z3_right * temp;
    z3_right = z2_right + z3_right;
    // Now the left leg...
    float x3_left = cos(hind_left_theta - hind_left_phi);
    x3_left = x3_left * F;
    temp = cos(hind_left_alpha + PI);           //Add PI to invert to get the sign correct
    x3_left = x3_left * temp;
    x3_left = x2_left + x3_left;
    float y3_left = sin(hind_left_theta + PI - hind_left_phi);
    y3_left = y3_left * F;
    y3_left = y2_left + y3_left;
    float z3_left = cos(hind_left_theta + PI - hind_left_phi);
    z3_left = z3_left * F;
    temp = sin(hind_left_alpha);
    z3_left = z3_left * temp;
    z3_left = z2_left + z3_left;    
 
    // Now send the values back...   
    Positions pos;              // Struct to hold positions
    pos.X_right = (int) x3_right;
    pos.Y_right = (int) y3_right;
    pos.Z_right = (int) z3_right;
    pos.X_left = (int) x3_left;
    pos.Y_left = (int) y3_left;
    pos.Z_left = (int) z3_left;
        
    return(pos);                // Return positions
}