/**********************************************************************
 This function puts E.R.I.C. into a standard position
 regardless of E.R.I.C.'s initial positioning
 Notebook: http://mbed.org/users/ms523/notebook/eric-initialisation-/
***********************************************************************/
#include "ERIC.h"

#define TIME 0.5000       // Time to complete the movement

void Set_Pose(const int *pose) {

    Positions StartPos = FK_Engine();     // Get the initial foot positions...
    // Apply global offsets to allow for hip width
    StartPos.X_right = StartPos.X_right + X_OFFSET;
    StartPos.X_left = StartPos.X_left - X_OFFSET;

    // Create variables to hold the set points
    int X_right_distance, Y_right_distance, Z_right_distance;
    int X_left_distance, Y_left_distance, Z_left_distance;

    // Calculate the distance to travel in each axis...
    // Equation: Distance = End Position - Start Position
    X_right_distance = pose[0] - StartPos.X_right;
    Y_right_distance = pose[1] - StartPos.Y_right;
    Z_right_distance = pose[2] - StartPos.Z_right;
    X_left_distance = pose[3] - StartPos.X_left;
    Y_left_distance = pose[4] - StartPos.Y_left;
    Z_left_distance = pose[5] - StartPos.Z_left;

    // Calculate Velocity for each axis...
    // Equation: Velcoity = Distance / Time
    float X_right_velocity = X_right_distance / TIME;
    float Y_right_velocity = Y_right_distance / TIME;
    float Z_right_velocity = Z_right_distance / TIME;
    float X_left_velocity = X_left_distance / TIME;
    float Y_left_velocity = Y_left_distance / TIME;
    float Z_left_velocity = Z_left_distance / TIME;

    // Create a timer to control the movement
    Timer Movement_timer;

    /***********************************************************
    Sart moving the legs to the 'Stand' pose
    ***********************************************************/
    // Create variables to hold the joint angles & current set points
    Joint_Angles Hind_left, Hind_right;                 // Variables for joint angles
    float Hind_right_X, Hind_right_Y, Hind_right_Z;     // Variables for the right leg
    float Hind_left_X, Hind_left_Y, Hind_left_Z;        // Variables for the left leg

    Movement_timer.start();                             // Start the movement timer
    float time = 0;                                     // Create a variable to read the time

    while (time < TIME) {
        // Get the current time
        time = Movement_timer.read();
        if (time > TIME)
            time = TIME;

        // Calculate current set points
        Hind_right_X = (time * X_right_velocity) + StartPos.X_right;
        Hind_right_Y = (time * Y_right_velocity) + StartPos.Y_right;
        Hind_right_Z = (time * Z_right_velocity) + StartPos.Z_right;
        Hind_left_X = (time * X_left_velocity) + StartPos.X_left;
        Hind_left_Y = (time * Y_left_velocity) + StartPos.Y_left;
        Hind_left_Z = (time * Z_left_velocity) + StartPos.Z_left;

        // Calculate the required joint angles...
        Hind_right = IK_Engine(Hind_right_X,Hind_right_Y,Hind_right_Z);
        Hind_left = IK_Engine(Hind_left_X,Hind_left_Y,Hind_left_Z);

        // Translate the joint angles into Servo Angles
        Hind_right.lateral = 165 - Hind_right.lateral;
        Hind_right.hip = 340 - Hind_right.hip;
        Hind_right.knee = 320 - Hind_right.knee;
        Hind_left.lateral = Hind_left.lateral + 135;
        Hind_left.hip = Hind_left.hip - 40;
        Hind_left.knee = Hind_left.knee - 20;

        // And finally move the leg to the current set point
        Right_Hip_Lateral.SetGoal((int)Hind_right.lateral);
        Right_Hip.SetGoal((int)Hind_right.hip);
        Right_Knee.SetGoal((int)Hind_right.knee);
        Left_Hip_Lateral.SetGoal((int)Hind_left.lateral);
        Left_Hip.SetGoal((int)Hind_left.hip);
        Left_Knee.SetGoal((int)Hind_left.knee);
    }
}